package javax.swing;

import java.util.EventListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.EventListenerList;
import java.io.Serializable;

public abstract class AbstractListModel<E> implements ListModel<E>, Serializable
{
    protected EventListenerList listenerList;
    
    public AbstractListModel() {
        this.listenerList = new EventListenerList();
    }
    
    @Override
    public void addListDataListener(ListDataListener l) {
        this.listenerList.add(ListDataListener.class, l);
    }
    
    @Override
    public void removeListDataListener(ListDataListener l) {
        this.listenerList.remove(ListDataListener.class, l);
    }
    
    public ListDataListener[] getListDataListeners() {
        return (ListDataListener[])this.listenerList.getListeners(ListDataListener.class);
    }
    
    protected void fireContentsChanged(Object source, int index0, int index2) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ListDataListener.class) {
                if (o == null) {
                    o = new ListDataEvent(source, 0, index0, index2);
                }
                ((ListDataListener)listenerList[i + 1]).contentsChanged(o);
            }
        }
    }
    
    protected void fireIntervalAdded(Object source, int index0, int index2) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ListDataListener.class) {
                if (o == null) {
                    o = new ListDataEvent(source, 1, index0, index2);
                }
                ((ListDataListener)listenerList[i + 1]).intervalAdded(o);
            }
        }
    }
    
    protected void fireIntervalRemoved(Object source, int index0, int index2) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ListDataListener.class) {
                if (o == null) {
                    o = new ListDataEvent(source, 2, index0, index2);
                }
                ((ListDataListener)listenerList[i + 1]).intervalRemoved(o);
            }
        }
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> t) {
        return this.listenerList.getListeners(t);
    }
}
